import { of } from 'rxjs';
import { filter, map, mergeMap, switchMap } from 'rxjs/operators';
import { Err, Ok } from './index';
export function resultMap(mapper) {
    return (source) => {
        return source.pipe(map(result => result.map(mapper)));
    };
}
export function resultMapErr(mapper) {
    return (source) => {
        return source.pipe(map(result => result.mapErr(mapper)));
    };
}
export function resultMapTo(value) {
    return (source) => {
        return source.pipe(map(result => result.map(() => value)));
    };
}
export function resultMapErrTo(value) {
    return (source) => {
        return source.pipe(map(result => result.mapErr(() => value)));
    };
}
export function elseMap(mapper) {
    return (source) => {
        return source.pipe(map(result => {
            if (result.err) {
                return mapper(result.val);
            }
            else {
                return result.val;
            }
        }));
    };
}
export function elseMapTo(value) {
    return (source) => {
        return source.pipe(map(result => {
            if (result.err) {
                return value;
            }
            else {
                return result.val;
            }
        }));
    };
}
export function resultSwitchMap(mapper) {
    return (source) => {
        return source.pipe(switchMap(result => {
            if (result.ok) {
                return mapper(result.val);
            }
            else {
                return of(result);
            }
        }), map(result => {
            if (result instanceof Ok || result instanceof Err) {
                return result;
            }
            else {
                return new Ok(result);
            }
        }));
    };
}
export function resultMergeMap(mapper) {
    return (source) => {
        return source.pipe(mergeMap(result => {
            if (result.ok) {
                return mapper(result.val);
            }
            else {
                return of(result);
            }
        }), map(result => {
            if (result instanceof Ok || result instanceof Err) {
                return result;
            }
            else {
                return new Ok(result);
            }
        }));
    };
}
export function filterResultOk() {
    return (source) => {
        return source.pipe(filter(result => result.ok), map(result => result.val));
    };
}
export function filterResultErr() {
    return (source) => {
        return source.pipe(filter(result => result.err), map(result => result.val));
    };
}
//# sourceMappingURL=rxjs-operators.js.map